import json


def format_us_address(address_data):
    """Helper function to format US address data for display"""
    if not address_data:
        return "No address"

    if isinstance(address_data, str):
        try:
            address_data = json.loads(address_data)
        except json.JSONDecodeError:
            return address_data

    if not isinstance(address_data, dict):
        return str(address_data)

    lines = []

    # Add street address
    street = address_data.get('street_address', '').strip()
    if street:
        lines.append(street)

    # Add street address 2 if present
    street2 = address_data.get('street_address_2', '').strip()
    if street2:
        lines.append(street2)

    # Add city, state, zip
    city = address_data.get('city', '').strip()
    state = address_data.get('state', '').strip()
    zip_code = address_data.get('zip_code', '').strip()

    if city or state or zip_code:
        city_state_zip = f"{city}, {state} {zip_code}".strip()
        if city_state_zip != ", ":
            lines.append(city_state_zip)

    return ' • '.join(lines) if lines else "No address entered"



import json
import os

from django import forms
from django.utils.safestring import mark_safe

from django_reusable.utils import xstr
from django_reusable.utils.address_utils import format_us_address


class FileLinkWidget(forms.TextInput):
    def __init__(self, base_path, base_url, attrs=None):
        self.base_path = base_path
        self.base_url = base_url
        super(FileLinkWidget, self).__init__(attrs)

    def get_link(self, value):
        if value and os.path.exists(os.path.join(self.base_path, value)):
            return mark_safe('<a href="%s" target="_blank" class="btn btn-default">Open</a>' %
                             os.path.join(self.base_url, value))
        return ''

    def render(self, name, value, attrs=None, renderer=None):
        html = '''
            <div class="input-group">
                %s
                <span class="input-group-btn"> 
                    %s
                </span>
            </div>
        ''' % (super().render(name, value, attrs, renderer), self.get_link(value))
        return html


class ReadonlySelect(forms.Select):
    def render(self, name, value, attrs=None, renderer=None):
        matches = [t for (v, t) in self.choices if v == value]
        text = matches[0] if matches else None
        return f'''{xstr(text, '---')}<input name="{name}" value="{value}" type="hidden" />'''


class ReadonlyMultiSelect(forms.SelectMultiple):
    def render(self, name, value, attrs=None, renderer=None):
        value_list = value if isinstance(value, list) or isinstance(value, tuple) else eval(value)
        matches = [t for (v, t) in self.choices if v in value_list]
        text = ', '.join(matches) if matches else None
        attrs = attrs or {}
        attrs['style'] = 'display: none;'
        default_widget = super().render(name, value, attrs, renderer)
        return f'''{xstr(text, '---')}{default_widget}'''


class ReadOnlyInput(forms.TextInput):
    def render(self, name, value, attrs=None, renderer=None):
        attrs = attrs or {}
        attrs['style'] = 'display: none;'
        return f'{xstr(value)}{super().render(name, value, attrs, renderer)}'


class DateInput(forms.DateInput):
    input_type = 'date'


class SingleCharSplitInput(forms.TextInput):
    def __init__(self, split=2, *args, **kwargs):
        self.split = split
        super().__init__(*args, **kwargs)

    def render(self, name, value, attrs=None, renderer=None):
        attrs = attrs or {}
        split_input_attrs = attrs.copy()
        split_input_attrs.update({
            'minlength': 1,
            'maxlength': 1,
            'style': 'width: 20px; margin-right: 3px;',
            'class': 'form-control split-char',
            'id': '',
        })
        attrs_str = " ".join([f'{k}="{v}"' for (k, v) in split_input_attrs.items()])
        inputs = [f'<input {attrs_str} />' for x in range(0, self.split)]
        attrs['style'] = 'display: none;'
        return '''
                <script>
                    $(document).ready(function () {
                        $('input.split-char').on("keyup", function(){
                            if($(this).val().length == $(this).attr("maxlength")){
                                $(this).next().focus();
                                $("#id_%s").val(Array.from($('input.split-char')).map(x => x.value).join(''));
                            }
                        }); 
                    })
                </script>
               ''' % (name,) + f'''    
            <div class='single-char-split-inputs'>
                <div>{"".join(inputs)}</div>
                {super().render(name, value, attrs, renderer)}
            </div>
        '''


class USAddressWidget(forms.Widget):
    """
    A widget that renders multiple input fields for US address components.
    """
    template_name = 'django_reusable/widgets/us_address_widget.html'

    def __init__(self, attrs=None):
        super().__init__(attrs)

    def format_value(self, value):
        """Format the value for display in the widget"""
        if value is None:
            return {
                'street_address': '',
                'street_address_2': '',
                'city': '',
                'state': '',
                'zip_code': ''
            }

        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.JSONDecodeError:
                return {
                    'street_address': '',
                    'street_address_2': '',
                    'city': '',
                    'state': '',
                    'zip_code': ''
                }

        if isinstance(value, dict):
            return {
                'street_address': value.get('street_address', ''),
                'street_address_2': value.get('street_address_2', ''),
                'city': value.get('city', ''),
                'state': value.get('state', ''),
                'zip_code': value.get('zip_code', '')
            }

        return {
            'street_address': '',
            'street_address_2': '',
            'city': '',
            'state': '',
            'zip_code': ''
        }

    def value_from_datadict(self, data, files, name):
        """Extract address data from form submission"""
        return {
            'street_address': data.get(f'{name}_street_address', ''),
            'street_address_2': data.get(f'{name}_street_address_2', ''),
            'city': data.get(f'{name}_city', ''),
            'state': data.get(f'{name}_state', ''),
            'zip_code': data.get(f'{name}_zip_code', '')
        }

    def get_context(self, name, value, attrs):
        """Get context for template rendering"""
        context = super().get_context(name, value, attrs)

        formatted_value = self.format_value(value)
        formatted_display = format_us_address(formatted_value)

        # US states list
        us_states = [
            ('', '-- State --'),
            ('AL', 'Alabama'), ('AK', 'Alaska'), ('AZ', 'Arizona'), ('AR', 'Arkansas'),
            ('CA', 'California'), ('CO', 'Colorado'), ('CT', 'Connecticut'), ('DE', 'Delaware'),
            ('FL', 'Florida'), ('GA', 'Georgia'), ('HI', 'Hawaii'), ('ID', 'Idaho'),
            ('IL', 'Illinois'), ('IN', 'Indiana'), ('IA', 'Iowa'), ('KS', 'Kansas'),
            ('KY', 'Kentucky'), ('LA', 'Louisiana'), ('ME', 'Maine'), ('MD', 'Maryland'),
            ('MA', 'Massachusetts'), ('MI', 'Michigan'), ('MN', 'Minnesota'), ('MS', 'Mississippi'),
            ('MO', 'Missouri'), ('MT', 'Montana'), ('NE', 'Nebraska'), ('NV', 'Nevada'),
            ('NH', 'New Hampshire'), ('NJ', 'New Jersey'), ('NM', 'New Mexico'), ('NY', 'New York'),
            ('NC', 'North Carolina'), ('ND', 'North Dakota'), ('OH', 'Ohio'), ('OK', 'Oklahoma'),
            ('OR', 'Oregon'), ('PA', 'Pennsylvania'), ('RI', 'Rhode Island'), ('SC', 'South Carolina'),
            ('SD', 'South Dakota'), ('TN', 'Tennessee'), ('TX', 'Texas'), ('UT', 'Utah'),
            ('VT', 'Vermont'), ('VA', 'Virginia'), ('WA', 'Washington'), ('WV', 'West Virginia'),
            ('WI', 'Wisconsin'), ('WY', 'Wyoming')
        ]

        context.update({
            'formatted_display': formatted_display,
            'us_states': us_states,
            'value': formatted_value,
        })

        return context


class CurrencyInput(forms.TextInput):
    """
    A widget that formats input as US currency with $ sign and commas on blur
    """

    class Media:
        js = ('django_reusable/js/currency-format.js',)

    def __init__(self, attrs=None):
        default_attrs = {'class': 'currency-input form-control'}
        if attrs:
            default_attrs.update(attrs)
        super().__init__(default_attrs)

    def format_value(self, value):
        """Format the value for display - keep as plain number for form processing"""
        if value is None or value == '':
            return ''
        try:
            # Just return the numeric value, let JavaScript handle the display formatting
            return str(float(value))
        except (ValueError, TypeError):
            return str(value) if value else ''



from django import forms
import json

from .widgets import USAddressWidget, CurrencyInput


class ChoiceFieldNoValidation(forms.ChoiceField):
    """
    An override of django's ChoiceField to ignore validation of the choices.
    Primarily used for cases where the choice fields are assigned options dynamically on the page.
    """

    def validate(self, value):
        pass


class CheckboxMultipleChoiceField(forms.MultipleChoiceField):
    widget = forms.CheckboxSelectMultiple

    def __init__(self, *args, **kwargs):
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)  # We use our own widget
        super().__init__(*args, **kwargs)

    def to_python(self, value):
        """Override to handle JSON string values properly"""
        if not value:
            return []

        # If it's already a list, return it
        if isinstance(value, list):
            return [str(item) for item in value]

        # If it's a JSON string, parse it
        if isinstance(value, str):
            try:
                parsed = json.loads(value)
                if isinstance(parsed, list):
                    return [str(item) for item in parsed]
                else:
                    return [str(parsed)]
            except json.JSONDecodeError:
                return [str(value)]

        return [str(value)]

    def validate(self, value):
        """Override validation to handle our custom data format"""
        if self.required and not value:
            raise forms.ValidationError(self.error_messages['required'], code='required')

        # Validate each choice
        if value and self.choices:
            valid_choices = [str(choice[0]) for choice in self.choices]
            for item in value:
                if str(item) not in valid_choices:
                    raise forms.ValidationError(
                        self.error_messages['invalid_choice'],
                        code='invalid_choice',
                        params={'value': item},
                    )

    def widget_attrs(self, widget):
        attrs = super().widget_attrs(widget)
        if isinstance(widget, forms.CheckboxSelectMultiple):
            attrs['class'] = (attrs.get('class', '') + ' unstyled').strip()
        return attrs


class USAddressFormField(forms.Field):
    """
    A form field for US addresses that renders as multiple input fields
    and validates the complete address data.
    """
    widget = USAddressWidget

    def __init__(self, *args, **kwargs):
        # Remove TextField-specific parameters that this form field doesn't use
        kwargs.pop('max_length', None)
        kwargs.pop('widget', None)  # We use our own widget
        super().__init__(*args, **kwargs)

    def to_python(self, value):
        """Convert form data to Python object"""
        if not value:
            return None

        if isinstance(value, dict):
            # Clean empty values
            cleaned_value = {}
            for key, val in value.items():
                if val and val.strip():
                    cleaned_value[key] = val.strip()

            # Return None if no meaningful data
            if not any(cleaned_value.values()):
                return None

            return cleaned_value

        return value

    def validate(self, value):
        """Validate the address data"""
        super().validate(value)

        if not value and self.required:
            raise forms.ValidationError("Address is required.")

        if not value:
            return

        # Validate required fields
        required_fields = {
            'street_address': 'Street address',
            'city': 'City',
            'state': 'State',
            'zip_code': 'ZIP code'
        }

        for field, display_name in required_fields.items():
            if field not in value or not value[field]:
                raise forms.ValidationError(f"{display_name} is required.")

        # Validate state format (2-letter code)
        if len(value['state']) != 2:
            raise forms.ValidationError("State must be a 2-letter code.")

        # Basic zip code validation
        zip_code = str(value['zip_code'])
        if not (len(zip_code) == 5 and zip_code.isdigit()) and not (
                len(zip_code) == 10 and zip_code[5] == '-' and
                zip_code[:5].isdigit() and zip_code[6:].isdigit()
        ):
            raise forms.ValidationError("ZIP code must be in format 12345 or 12345-6789.")

    def prepare_value(self, value):
        """Prepare value for display in the widget"""
        if isinstance(value, str):
            try:
                return json.loads(value)
            except json.JSONDecodeError:
                return None
        return value


class CurrencyFormField(forms.DecimalField):
    def __init__(self, *args, **kwargs):
        kwargs['max_digits'] = kwargs.get('max_digits', 10)
        kwargs['decimal_places'] = kwargs.get('decimal_places', 2)
        kwargs['widget'] = kwargs.get('widget', CurrencyInput())
        super().__init__(*args, **kwargs)



import json
from django.db import models
from django.core.exceptions import ValidationError

from .lookups import JSONExtract
from ..utils.address_utils import format_us_address
from django_reusable.forms.fields import USAddressFormField, CheckboxMultipleChoiceField, CurrencyFormField


class MultipleChoiceField(models.TextField):
    """
    A Django model field for storing multiple choice selections in JSON format.

    The field stores selected values as a JSON array:
    ["choice1", "choice2", "choice3"]
    """

    description = "A field for storing multiple choice selections as JSON array"

    def __init__(self, choices=None, *args, **kwargs):
        # Pass choices to the parent class so Django handles them properly
        if choices is not None:
            kwargs['choices'] = choices
        super().__init__(*args, **kwargs)

    def from_db_value(self, value, expression, connection):
        """Convert database value to Python object"""
        if value is None:
            return []
        if isinstance(value, str):
            try:
                result = json.loads(value)
                return result if isinstance(result, list) else []
            except json.JSONDecodeError:
                return []
        return value if isinstance(value, list) else []

    def to_python(self, value):
        """Convert value to Python object"""
        if isinstance(value, list):
            return value
        if value is None:
            return []
        if isinstance(value, str):
            try:
                result = json.loads(value)
                return result if isinstance(result, list) else []
            except json.JSONDecodeError:
                return []
        return []

    def get_prep_value(self, value):
        """Convert Python object to database value"""
        if value is None:
            return json.dumps([])
        if isinstance(value, list):
            return json.dumps(value)
        return json.dumps([])

    def validate(self, value, model_instance):
        """Validate the choice selections"""

        if value is None:
            return

        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.JSONDecodeError:
                raise ValidationError("Invalid JSON format for multiple choice field")

        if not isinstance(value, list):
            raise ValidationError("Multiple choice field must be a list")

        # Validate choices if defined
        if self.choices:
            valid_choices = [choice[0] for choice in self.choices]
            for selected_value in value:
                if selected_value not in valid_choices:
                    raise ValidationError(f"'{selected_value}' is not a valid choice")

    def contribute_to_class(self, cls, name, **kwargs):
        """Called when the field is added to a model class"""
        super().contribute_to_class(cls, name, **kwargs)

        # Add a method to format the address for display
        def get(self):
            try:
                return ', '.join(getattr(self, name))
            except Exception:
                return str(getattr(self, name))

        get.__name__ = name
        setattr(cls, f'get_{name}_display', get)

    def formfield(self, **kwargs):
        # Don't call super().formfield() as TextField will override our form field
        # Instead, directly instantiate our custom form field
        defaults = {
            'choices': self.choices,
            'required': not self.blank,
            'initial': self.get_default(),
        }
        defaults.update(kwargs)

        # Return the form field directly without calling super()
        return CheckboxMultipleChoiceField(**defaults)


class USAddressField(models.TextField):
    """
    A Django model field for storing US addresses in JSON format.

    The field stores address data as a JSON object with the following structure:
    {
        "street_address": "123 Main St",
        "street_address_2": "Apt 4B",  # Optional
        "city": "New York",
        "state": "NY",
        "zip_code": "10001"
    }

    Supports querying by individual properties:
    - address__street_address='123 Main St'
    - address__city='New York'
    - address__state='NY'
    - address__zip_code='10001'
    """

    description = "A field for storing US addresses in JSON format"
    class_lookups = {
        'street_address': JSONExtract.create_instance('street_address'),
        'street_address_2': JSONExtract.create_instance('street_address_2'),
        'city': JSONExtract.create_instance('city'),
        'state': JSONExtract.create_instance('state'),
        'zip_code': JSONExtract.create_instance('zip_code'),
    }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def from_db_value(self, value, expression, connection):
        """Convert database value to Python object"""
        if value is None:
            return value
        if isinstance(value, str):
            try:
                return json.loads(value)
            except json.JSONDecodeError:
                return value
        return value

    def to_python(self, value):
        """Convert value to Python object"""
        if isinstance(value, dict):
            return value
        if value is None:
            return value
        if isinstance(value, str):
            try:
                return json.loads(value)
            except json.JSONDecodeError:
                return value
        return value

    def get_prep_value(self, value):
        """Convert Python object to database value"""
        if value is None:
            return value
        if isinstance(value, dict):
            return json.dumps(value)
        return value

    def validate(self, value, model_instance):
        """Validate the address data"""
        super().validate(value, model_instance)

        if value is None:
            return

        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.JSONDecodeError:
                raise ValidationError("Invalid JSON format for address")

        if not isinstance(value, dict):
            raise ValidationError("Address must be a dictionary")

        # Validate required fields
        required_fields = ['street_address', 'city', 'state', 'zip_code']
        for field in required_fields:
            if field not in value or not value[field]:
                raise ValidationError(f"Address field '{field}' is required")

        # Validate state format (2-letter code)
        if len(value['state']) != 2:
            raise ValidationError("State must be a 2-letter code")

        # Basic zip code validation
        zip_code = str(value['zip_code'])
        if not (len(zip_code) == 5 or (len(zip_code) == 10 and zip_code[5] == '-')):
            raise ValidationError("Zip code must be in format XXXXX or XXXXX-XXXX")

    def contribute_to_class(self, cls, name, **kwargs):
        """Called when the field is added to a model class"""
        super().contribute_to_class(cls, name, **kwargs)

        # Add a method to format the address for display
        def format_address(self):
            address_value = getattr(self, name)
            return format_us_address(address_value)

        format_address.__name__ = name
        setattr(cls, f'get_{name}_display', format_address)

    def formfield(self, **kwargs):

        # Set our custom form field as the default
        defaults = {'form_class': USAddressFormField}
        defaults.update(kwargs)

        # Call the parent's formfield method with our custom form class
        return super().formfield(**defaults)


class CurrencyField(models.DecimalField):
    """
    A Django model field for storing currency values with fixed precision.

    The field uses DecimalField with decimal_places=2 by default.
    """

    description = "A field for storing currency values"

    def __init__(self, *args, **kwargs):
        # Set default max_digits and decimal_places if not provided
        kwargs.setdefault('decimal_places', 2)
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        # Set our custom form field as the default
        defaults = {'form_class': CurrencyFormField}
        defaults.update(kwargs)

        # Call the parent's formfield method with our custom form class
        return super().formfield(**defaults)



from django.db.models import Transform, Lookup


class JSONExtract(Transform):
    """
    A Django model lookup for extracting and performing case-insensitive
    comparisons on JSON properties stored as JSON in field.
    Used for lookups like:
    - field__json_property='value'
    - field__json_property__iexact='value'
    - field__json_property__icontains='value'
    - field__json_property__exact='value'
    - field__json_property__contains='value'
    """
    @staticmethod
    def _get_sub(_lookup_name, expr):
        class JSONExtractSub(Lookup):
            """Case-insensitive lookup for JSON extracted values"""
            lookup_name = _lookup_name

            def as_sql(self, compiler, connection):
                lhs, lhs_params = self.process_lhs(compiler, connection)
                rhs, rhs_params = self.process_rhs(compiler, connection)

                # For LIKE operations, wrap the value with wildcards
                if expr == 'LIKE':
                    # Escape the rhs value and add wildcards
                    if rhs_params:
                        # Convert to lowercase and add wildcards for LIKE pattern
                        rhs_params = [f"%{str(rhs_params[0]).lower()}%"]
                    params = lhs_params + rhs_params
                    return f'{lhs} {expr} %s', params
                else:
                    # For exact matches, just lowercase the rhs
                    if rhs_params:
                        rhs_params = [str(rhs_params[0]).lower()]
                    params = lhs_params + rhs_params
                    return f'{lhs} {expr} %s', params

        return JSONExtractSub

    """Base class for extracting JSON properties"""

    @classmethod
    def create_instance(cls, json_path):
        class JSONExtractInstance(JSONExtract):
            lookup_name = json_path

            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.json_path = json_path

        return JSONExtractInstance

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.json_path = None
        self.__class__.class_lookups = {
            'exact': self._get_sub('exact', '='),
            'iexact': self._get_sub('iexact', '='),
            'contains': self._get_sub('contains', 'LIKE'),
            'icontains': self._get_sub('icontains', 'LIKE'),
        }

    def as_sql(self, compiler, connection):
        lhs, params = compiler.compile(self.lhs)
        # Use JSON extraction for SQLite (json_extract function)
        # For other databases, this might need adjustment
        # Wrap with LOWER() for case-insensitive comparisons
        if connection.vendor == 'sqlite':
            return f"LOWER(json_extract({lhs}, '$.{self.json_path}'))", params
        elif connection.vendor == 'postgresql':
            return f"LOWER({lhs}->>'%s')" % self.json_path, params
        elif connection.vendor == 'mysql':
            return f"LOWER(JSON_UNQUOTE(JSON_EXTRACT({lhs}, '$.{self.json_path}')))", params
        else:
            # Fallback: try to parse JSON in Python (less efficient)
            return f"LOWER({lhs})", params



# Generated by Django 2.0.5 on 2025-08-28 00:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_reusable', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='error',
            options={'ordering': ['-last_seen']},
        ),
    ]



from django.contrib import admin
from django.contrib.admin import FieldListFilter


class TextInputFilter(admin.SimpleListFilter):
    template = 'django_reusable/filters/input-filter.html'

    def lookups(self, request, model_admin):
        # Dummy, required to show the filter.
        return (('Dummy', 'Dummy'),)

    def queryset(self, request, queryset):
        return queryset


class SearchInFilter(admin.SimpleListFilter):
    title = 'Search In'
    parameter_name = 'search'
    lookup_choices = []

    def lookups(self, request, model_admin):
        return self.lookup_choices

    def queryset(self, request, queryset):
        return queryset


class MultiSelectFilterMixin:
    """
    Base mixin for multi-select admin filters using <select multiple>.

    Provides common functionality for filters that allow multiple values
    to be selected and filtered with OR logic (field__in lookup).
    """

    template = 'django_reusable/filters/multi-select-filter.html'

    def expected_parameters(self):
        """Return list of expected URL parameters for this filter"""
        return [self.parameter_name]

    def get_selected_values(self):
        """Get list of currently selected values from URL parameters"""
        return self.used_parameters.get(self.parameter_name, [])

    def queryset(self, request, queryset):
        """Apply filtering with __in lookup for selected values"""
        values = self.get_selected_values()
        if values:
            lookup_field = self.field_path
            try:
                return queryset.filter(**{f'{lookup_field}__in': values})
            except Exception as e:
                # Invalid values - return unfiltered queryset
                return queryset
        return queryset

    @property
    def options(self):
        """Get all available options for the select element"""
        return self.lookups(self.request, self.model_admin)

    @property
    def selected_values(self):
        """Get currently selected values"""
        return self.get_selected_values()

    @property
    def other_params(self):
        """Get other GET parameters (excluding this filter's parameter)"""
        # Use request.GET which is a QueryDict that preserves multiple values
        params = self.request.GET.copy()
        params.pop(self.parameter_name, None)
        return params

    @property
    def clear_url(self):
        """Build URL to clear this filter"""
        # Use request.GET to preserve multiple values in other parameters
        params = self.request.GET.copy()
        params.pop(self.parameter_name, None)
        if params:
            return '?' + params.urlencode()
        return '?'

    def choices(self, changelist):
        """
        For compatibility with FieldListFilter.
        Return empty generator since we use a custom template.
        """
        return iter([])


class MultiSelectAllValuesFieldFilter(MultiSelectFilterMixin, FieldListFilter):
    """
    Multi-select filter for all distinct values in a model field.

    Usage:
        class MyAdmin(admin.ModelAdmin):
            list_filter = [
                ('field_name', MultiSelectAllValuesFieldFilter),
            ]
    """

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.field = field
        self.field_path = field_path
        self.parameter_name = f'{field_path}__in'
        self.title = field.verbose_name
        self.request = request
        self.model_admin = model_admin
        self.params = params

        # Call parent init first
        super().__init__(field, request, params, model, model_admin, field_path)

        # Extract selected values AFTER super().__init__() to avoid it being overwritten
        # Always use request.GET.getlist() for multiple values
        self.used_parameters = {}
        if self.parameter_name in request.GET:
            value = request.GET.getlist(self.parameter_name)
            # Ensure all values are strings for proper comparison in template
            value = [str(v) for v in value if v]
            if value:
                self.used_parameters[self.parameter_name] = value

    def lookups(self, request, model_admin):
        """Get all distinct values from database"""
        qs = model_admin.get_queryset(request)
        values = qs.order_by(self.field.name).values_list(
            self.field.name, flat=True
        ).distinct()

        # Return (value, label) tuples
        return [(str(val), str(val)) for val in values if val is not None]

    def has_output(self):
        return True


class MultiSelectChoicesFieldFilter(MultiSelectFilterMixin, FieldListFilter):
    """
    Multi-select filter for fields with predefined choices.

    Works with fields where value != label, like:
        position = models.CharField(
            choices=[('SE', 'Software Engineer'), ('MGR', 'Manager')]
        )

    Usage:
        class MyAdmin(admin.ModelAdmin):
            list_filter = [
                ('position', MultiSelectChoicesFieldFilter),
            ]
    """

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.field = field
        self.field_path = field_path
        self.parameter_name = f'{field_path}__in'
        self.title = field.verbose_name
        self.lookup_choices = field.choices or []
        self.request = request
        self.model_admin = model_admin
        self.params = params

        # Call parent init first
        super().__init__(field, request, params, model, model_admin, field_path)

        # Extract selected values AFTER super().__init__() to avoid it being overwritten
        # Always use request.GET.getlist() for multiple values
        self.used_parameters = {}
        if self.parameter_name in request.GET:
            value = request.GET.getlist(self.parameter_name)
            # Ensure all values are strings for proper comparison in template
            value = [str(v) for v in value if v]
            if value:
                self.used_parameters[self.parameter_name] = value

    def lookups(self, request, model_admin):
        """Return field's defined choices"""
        return self.lookup_choices

    def has_output(self):
        return True


class MultiSelectRelatedFieldFilter(MultiSelectFilterMixin, FieldListFilter):
    """
    Multi-select filter for ForeignKey and ManyToManyField relationships.

    Usage:
        class MyAdmin(admin.ModelAdmin):
            list_filter = [
                ('author', MultiSelectRelatedFieldFilter),
            ]
    """

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.field = field
        self.field_path = field_path
        self.parameter_name = f'{field_path}__in'
        self.title = field.verbose_name
        self.request = request
        self.model_admin = model_admin
        self.params = params

        # Get related model
        if hasattr(field, 'related_model'):
            self.related_model = field.related_model
        else:
            self.related_model = field.remote_field.model

        # Call parent init first
        super().__init__(field, request, params, model, model_admin, field_path)

        # Extract selected values AFTER super().__init__() to avoid it being overwritten
        # Always use request.GET.getlist() for multiple values
        self.used_parameters = {}
        if self.parameter_name in request.GET:
            value = request.GET.getlist(self.parameter_name)
            # Ensure all values are strings for proper comparison in template
            value = [str(v) for v in value if v]
            if value:
                self.used_parameters[self.parameter_name] = value

    def lookups(self, request, model_admin):
        """Query related model for available objects"""
        qs = self.related_model.objects.all()

        # Respect any custom queryset from model_admin
        if hasattr(model_admin, 'get_field_queryset'):
            custom_qs = model_admin.get_field_queryset(None, self.field, request)
            if custom_qs:
                qs = custom_qs

        # Return (pk, str) tuples
        return [(str(obj.pk), str(obj)) for obj in qs]

    def queryset(self, request, queryset):
        """Apply filtering with proper type conversion for PKs"""
        values = self.get_selected_values()
        if values:
            try:
                # Convert string PKs to integers
                pk_list = [int(v) for v in values]
                return queryset.filter(**{f'{self.field_path}__in': pk_list})
            except (ValueError, TypeError):
                return queryset
        return queryset

    def has_output(self):
        return True
